@@ -4,7 +4,7 @@
 # Author      : E.Taskesen
 # Contact     : erdogant@gmail.com
 # github      : https://github.com/erdogant/treeplot
-# Licence     : MIT
+# Licence     : See Licences
 # --------------------------------------------------
 
 # %% Libraries
@@ -13,11 +13,13 @@ import sys
 import zipfile
 import numpy as np
 from sklearn.tree import export_graphviz
-from sklearn.datasets import make_classification
+from sklearn.tree.export import export_text
 from subprocess import call
 import matplotlib.image as mpimg
 import matplotlib.pyplot as plt
 from graphviz import Source
+import wget
+URL = 'https://erdogant.github.io/datasets/graphviz-2.38.zip'
 
 
 # %% Plot tree
@@ -40,14 +42,19 @@ def plot(model, featnames=None, num_trees=0, figsize=(25,25), verbose=3):
 
     Returns
     -------
-    None.
+    ax : Figure axis
+        Figure axis of the input model.
 
     """
     modelname=str(model).lower()
-    if ('tree' in modelname) or ('forest' in modelname):
-        ax=randomforest(model, featnames=featnames, num_trees=num_trees, figsize=figsize, verbose=verbose)
     if ('xgb' in modelname):
+        if verbose>=4: print('xgboost plotting pipeline.')
         ax=xgboost(model, featnames=featnames, num_trees=num_trees, figsize=figsize, verbose=verbose)
+    elif ('tree' in modelname) or ('forest' in modelname) or ('gradientboosting' in modelname):
+        if verbose>=4: print('tree plotting pipeline.')
+        ax=randomforest(model, featnames=featnames, num_trees=num_trees, figsize=figsize, verbose=verbose)
+    else:
+        print('[treeplot] Model %s not recognized.' %(modelname))
 
     return(ax)
 
@@ -74,7 +81,8 @@ def xgboost(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(
 
     Returns
     -------
-    ax.
+    ax : Figure axis
+        Figure axis of the input model.
 
     """
     try:
@@ -127,24 +135,33 @@ def randomforest(model, featnames=None, num_trees=0, filepath='tree', export='pn
 
     Returns
     -------
-    ax.
+    ax : Figure axis
+        Figure axis of the input model.
 
     """
     ax=None
+    dotfile = None
+    pngfile = None
+    # Check model
     _check_model(model, 'randomforest')
     # Set envirerement
     _set_graphviz_path()
-    dotfile = filepath + '.dot'
-    pngfile = filepath + '.png'
+    
+    if export is not None:
+        dotfile = filepath + '.dot'
+        pngfile = filepath + '.png'
 
-    if isinstance(featnames, type(None)):
+    if featnames is None:
         featnames = np.arange(0,len(model.feature_importances_)).astype(str)
 
     # Get model parameters
-    if hasattr(model, 'estimators_'):
-        estimator = model.estimators_[num_trees]
+    if ('gradientboosting' in str(model).lower()):
+        estimator = model.estimators_[num_trees][0]
     else:
-        estimator = model
+        if hasattr(model, 'estimators_'):
+            estimator = model.estimators_[num_trees]
+        else:
+            estimator = model
 
     # Make dot file
     dot_data = export_graphviz(estimator,
@@ -161,28 +178,36 @@ def randomforest(model, featnames=None, num_trees=0, filepath='tree', export='pn
     if export == 'pdf':
         s = Source.from_file(dotfile)
         s.view()
-
     # Save to png
-    if export == 'png':
+    elif export == 'png':
         try:
             call(['dot', '-Tpng', dotfile, '-o', pngfile, '-Gdpi=' + str(resolution)])
             fig, ax = plt.subplots(1, 1, figsize=figsize)
             img = mpimg.imread(pngfile)
             plt.imshow(img)
             plt.axis('off')
+            plt.show()
         except:
             if _get_platform() != "windows":
                 print('[TREEPLOT] Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
+    else:
+        graph = Source(dot_data)
+        plt.show()
 
     return(ax)
 
 
-# %% Example data
-def import_example(n_samples=1000, n_feat=10):
-    """Import Example.
+# %% Import example dataset from github.
+def import_example(data='random', n_samples=1000, n_feat=10):
+    """Import example dataset from sklearn.
 
     Parameters
     ----------
+    data : str
+        'random' : str, two-class
+        'breast' : str, two-class
+        'titanic': str, two-class
+        'iris' : str, multi-class
     n_samples : int, optional
         Number of samples to generate. The default is 1000.
     n_feat : int, optional
@@ -190,18 +215,36 @@ def import_example(n_samples=1000, n_feat=10):
 
     Returns
     -------
-    None.
+    tuple (X,y).
+        X is the dataset and y the response variable.
 
     """
-    [X, y] = make_classification(n_samples=n_samples, n_features=n_feat)
-    return(X,y)
+    try:
+        from sklearn import datasets
+    except:
+        print('This requires: <pip install sklearn>')
+        return None, None
+
+    if data=='iris':
+        X, y = datasets.load_iris(return_X_y=True)
+    elif data=='breast':
+        X, y = datasets.load_breast_cancer(return_X_y=True)
+    elif data=='titanic':
+        X, y = datasets.fetch_openml("titanic", version=1, as_frame=True, return_X_y=True)
+    elif data=='random':
+        X, y = datasets.make_classification(n_samples=n_samples, n_features=n_feat)
+
+    return X, y
 
 
 # %% Get graphiz path and include into local PATH
-def _set_graphviz_path(gfile='graphviz-2.38.zip', verbose=3):
+def _set_graphviz_path(verbose=3):
     finPath=''
     if _get_platform()=="windows":
-        curpath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'RESOURCES')
+        # Download from github
+        [gfile, curpath] = _download_graphviz(URL, verbose=verbose)
+
+        # curpath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'RESOURCES')
         # filesindir = os.listdir(curpath)[0]
         idx = gfile[::-1].find('.') + 1
         dirname = gfile[:-idx]
@@ -234,6 +277,7 @@ def _set_graphviz_path(gfile='graphviz-2.38.zip', verbose=3):
     return(finPath)
 
 
+# %%
 def _get_platform():
     platforms = {
         'linux1':'linux',
@@ -250,10 +294,41 @@ def _get_platform():
 def _check_model(model, expected):
     modelname = str(model).lower()
     if (expected=='randomforest'):
-        if ('forest' in modelname) or ('tree' in modelname):
+        if ('forest' in modelname) or ('tree' in modelname) or ('gradientboosting' in modelname):
             pass
         else:
             print('WARNING: The input model seems not to be a tree-based model?')
     if (expected=='xgb'):
         if ('xgb' not in modelname):
             print('WARNING: The input model seems not to be a xgboost model?')
+
+
+# %% Import example dataset from github.
+def _download_graphviz(url, verbose=3):
+    """Import example dataset from github.
+
+    Parameters
+    ----------
+    url : str, optional
+        url-Link to graphviz. The default is 'https://erdogant.github.io/datasets/graphviz-2.38.zip'.
+    verbose : int, optional
+        Print message to screen. The default is 3.
+
+    Returns
+    -------
+    tuple : (gfile, curpath).
+        gfile : filename
+        curpath : currentpath
+
+    """
+    curpath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'RESOURCES')
+    gfile = wget.filename_from_url(url)
+    PATH_TO_DATA = os.path.join(curpath, gfile)
+
+    # Check file exists.
+    if not os.path.isfile(PATH_TO_DATA):
+        # Download data from URL
+        if verbose>=3: print('[treeplot] Downloading graphviz..')
+        wget.download(url, curpath)
+
+    return(gfile, curpath)
